﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

using System.Reflection;

using Steamworks;

//credit to Steamworks.NET & SteamWrapper

namespace qc_uploader
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            if (!Initialize())
            {
                Environment.Exit(0);
                return;               
            }

            this.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.MyMainForm_FormClosing);
            Application.Idle += new EventHandler(Application_Idle);

            InitializeComponent();

            listbox_mods.AutoResizeColumns(ColumnHeaderAutoResizeStyle.None);

            

            modInfos = new List<ModInfo>();
            PopulateModList();
        }

        //make sure the player is logged in right & steam is running.
        bool Initialize()
        {
            if (!SteamAPI.Init())
            {
                MessageBox.Show("Failed to initialize Steam. Please make sure Steam is running.\n\nExiting.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            if (!SteamAPI.IsSteamRunning())
            {
                MessageBox.Show("Steam is not running. Please make sure Steam is running.\n\nExiting.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            
            return true;
        }


        

        void Application_Idle(object sender, EventArgs e)
        {
            SteamAPI.RunCallbacks();
        }

        private void MyMainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            SteamAPI.Shutdown();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void steamWorkshopLicenseAgreementToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("https://steamcommunity.com/login/home/?goto=sharedfiles%2Fworkshoplegalagreement");            
        }

        private void quadrilateralCowboyWorkshopToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(string.Format("http://steamcommunity.com/app/{0}/workshop", Constants.appID));
        }

        private void button2_Click(object sender, EventArgs e)
        {
            //click the ADD NEW MOD button.
            EditMod(true);
        }

        private void button_updatemod_Click(object sender, EventArgs e)
        {
            //click the EDIT MOD button.
            EditMod(false);
        }

        //isCreating true = is creating a new mod.
        //isCreating false = is editing an existing mod.
        private void EditMod(bool isCreating)
        {
            editscreen newEditscreen = new editscreen();
            newEditscreen.SetCreating(isCreating);
            newEditscreen.FormClosing += new FormClosingEventHandler(editscreen_closing);

            if (!isCreating)
            {
                //edit an existing mod.

                //determine which item is selected.
                int selectedModIndex = GetSelectedMod();

                if (selectedModIndex >= 0)
                {
                    newEditscreen.SetExistingInfo(modInfos[selectedModIndex]);
                }
                else
                {
                    //this should never happen but let's handle it just in case.
                    MessageBox.Show("Please select a mod from the above list.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
            else
            {
                newEditscreen.SetExistingInfo(new ModInfo()); //give it a blank modinfo so all information will be validated as new.
            }

            Enabled = false; //hack to make it look disabled.
            newEditscreen.ShowDialog();
            Enabled = true;  //hack to make it look disabled.
        }

        void editscreen_closing(object sender, FormClosingEventArgs e)
        {
            //when the editscreen closes, then update this list.
            listbox_mods.Items.Clear();
            modInfos.Clear();
            PopulateModList();
        }


        private int GetSelectedMod()
        {
            if (listbox_mods.SelectedItems.Count <= 0)
                return -1;

            ListViewItem i = listbox_mods.SelectedItems[0];

            if (i.Index >= 0)
                return i.Index;

            return -1;
        }

        private void listbox_mods_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (GetSelectedMod() < 0)
            {
                button_updatemod.Enabled = false;
            }
            else
            {
                button_updatemod.Enabled = true;
            }
        }






        // ===========================================================================

        // populate the mod list.

        // ===========================================================================

        List<ModInfo> modInfos;
        private UGCQueryHandle_t m_UGCQueryHandle;
        private CallResult<SteamUGCQueryCompleted_t> OnSteamUGCQueryCompletedCallResult;

        public void PopulateModList()
        {
            m_UGCQueryHandle = SteamUGC.CreateQueryUserUGCRequest(SteamUser.GetSteamID().GetAccountID(),
                EUserUGCList.k_EUserUGCList_Published, EUGCMatchingUGCType.k_EUGCMatchingUGCType_Items,
                EUserUGCListSortOrder.k_EUserUGCListSortOrder_CreationOrderDesc, SteamUtils.GetAppID(), Constants.GetAppID(), 1);

            OnSteamUGCQueryCompletedCallResult = CallResult<SteamUGCQueryCompleted_t>.Create(OnSteamUGCQueryCompleted);
            SteamAPICall_t handle = SteamUGC.SendQueryUGCRequest(m_UGCQueryHandle);
            OnSteamUGCQueryCompletedCallResult.Set(handle);
        }

        void OnSteamUGCQueryCompleted(SteamUGCQueryCompleted_t pCallback, bool bIOFailure)
        {
            if (pCallback.m_eResult != EResult.k_EResultOK || bIOFailure)
            {
                MessageBox.Show("Failed to retrieve mod list.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            uint totalMods = pCallback.m_unTotalMatchingResults;

            if (totalMods <= 0)
            {
                groupBox1.Text = string.Format("You have no published mods. Press 'Add new mod' to publish a mod.", SteamFriends.GetPersonaName());
                return;
            }


            //iterate through all of this player's published mods.

            for (uint i = 0; i < totalMods; i++)
            {
                SteamUGCDetails_t details;
                bool ret = SteamUGC.GetQueryUGCResult(m_UGCQueryHandle, i, out details);

                if (!ret)
                    continue;
                

                //Get the data from this mod.

                ModInfo newModInfo = new ModInfo();
                newModInfo.name = details.m_rgchTitle;
                newModInfo.lastUpdatedTime = Constants.FromUnixTime(details.m_rtimeUpdated);

                if (details.m_eVisibility == ERemoteStoragePublishedFileVisibility.k_ERemoteStoragePublishedFileVisibilityFriendsOnly)
                    newModInfo.visibility = 1;
                else if (details.m_eVisibility == ERemoteStoragePublishedFileVisibility.k_ERemoteStoragePublishedFileVisibilityPrivate)
                    newModInfo.visibility = 2;
                else
                    newModInfo.visibility = 0;
                
                newModInfo.publishedFileID = details.m_nPublishedFileId;
                newModInfo.description = details.m_rgchDescription;
                newModInfo.timeCreated = details.m_rtimeCreated;
                newModInfo.previewFilesize = details.m_nPreviewFileSize;

                
                //tags. Tags are one comma-separated string.
                string[] tags = details.m_rgchTags.Split(',');
                for (int k = 0; k < tags.Length; k++)
                {
                    newModInfo.tags.Add(tags[k]);
                }
                



                

                //add to internal list of mods.
                modInfos.Add(newModInfo);


                //add to the 'listbox_mods' display list.
                ListViewItem newItem = new ListViewItem();
                newItem.Text = newModInfo.name;

                if (newModInfo.visibility == 1)
                {
                    newItem.SubItems.Add("Friends-only");
                }
                else if (newModInfo.visibility == 2)
                {
                    newItem.SubItems.Add("Private");
                }
                else
                {
                    //0
                    newItem.SubItems.Add("Public");
                }

                DateTime dateOnly = newModInfo.lastUpdatedTime.Date;                
                newItem.SubItems.Add(dateOnly.ToString("d"));

                listbox_mods.Items.Add(newItem);
            }

            if (listbox_mods.Items.Count > 0)
            {
                listbox_mods.Items[0].Selected = true;
                button_updatemod.Enabled = true;

                groupBox1.Text = string.Format("{0}'s published mods", SteamFriends.GetPersonaName());
            }
        }

        

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var linkTimeLocal = Assembly.GetExecutingAssembly().GetLinkerTime();
            string text = string.Format("Workshop Uploader\nby Blendo Games\nApril 2016\n\nBuild:\n{0}\n\nLibrary used:\nSteamworks.NET\nhttps://steamworks.github.io", linkTimeLocal);

            MessageBox.Show(text, "About", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }




    }

    public class ModInfo
    {
        public ModInfo()
        {
            tags = new List<string>();
        }

        public PublishedFileId_t publishedFileID;
        public DateTime lastUpdatedTime;
        public string name;
        public string description;
        public int visibility;
        public List<string> tags;

        public uint timeCreated;
        public int previewFilesize;
    }
}
