﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Linq;
using System.IO;
using Microsoft.Win32;

using Steamworks;

namespace qc_uploader
{
    public partial class editscreen : Form
    {
        bool IsCreating;
        DateTime startTime;
        ModInfo originalInfo;
        

        public editscreen()
        {
            InitializeComponent();

            //checkedListBox1.ItemCheck += new ItemCheckEventHandler(checkedListBox1_ItemCheck);
            loadingpic.Hide();
        }



        public void SetExistingInfo(ModInfo modInfo)
        {
            textbox_name.Text = modInfo.name;
            textbox_description.Text = modInfo.description;
            combobox_visibility.SelectedIndex = (int)modInfo.visibility;


            //tags.
            for (int i = 0; i < modInfo.tags.Count; i++)
            {
                for (int k = 0; k < listbox_tags.Items.Count; k++)
                {
                    string listTag = listbox_tags.GetItemText(listbox_tags.Items[k]);

                    if (string.Compare(modInfo.tags[i], listTag, true) == 0)
                    {
                        listbox_tags.SetItemChecked(k, true);
                    }
                }
            }

            if (!IsCreating)
            {
                SetDebugLog(modInfo.name);
                SetDebugLog(string.Format("Original creation date: {0}", Constants.FromUnixTime(modInfo.timeCreated)));
                SetDebugLog(string.Format("Thumbnail image filesize: {0}", modInfo.previewFilesize));
            }
            


            publishedFileID = modInfo.publishedFileID;

            //hack to deselect everything.
            textbox_name.Select(0, 0);
            label1.Select();

            //store the info.
            originalInfo = modInfo;
        }

        public void SetCreating(bool value)
        {
            IsCreating = value;

            if (value)
            {
                button_publishmod.Text = "Add mod to Workshop";
                this.Text = "Add new mod";
            }
            else
            {
                button_publishmod.Text = "Upload changes";
                this.Text = "Update mod";
            }
        }

        private void label1_Click(object sender, EventArgs e) {}
        private void label4_Click(object sender, EventArgs e) { }

        private void button_cancelpublish_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //image browse button.
            OpenFileDialog dlg = new OpenFileDialog();

            dlg.Title = "Open Image";
            dlg.Filter = "Image files (*.jpg, *.png) | *.jpg; *.png|" + "All files|*.*";

            string qcFolder = GetQCFolder();
            if (Directory.Exists(qcFolder))
            {
                dlg.InitialDirectory = qcFolder;
            }

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    pictureBox1.Image = new Bitmap(dlg.FileName);
                    textbox_thumbnail.Text = dlg.FileName;
                }
                catch
                {
                    MessageBox.Show("There was a problem loading the image.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            dlg.Dispose();
        }


        //Locate the folder that QC is located in. Windows only for now
        private string GetQCFolder()
        {
            //get the steam install folder.
            Object steampath;
            String RegID = "SOFTWARE\\Valve\\Steam\\";
            RegistryKey key = Registry.CurrentUser;
            key = key.OpenSubKey(RegID);
            steampath = key.GetValue("SteamPath");
            steampath = steampath + "\\steamapps\\";
            string steaminstallpath = steampath.ToString();
            steaminstallpath = steaminstallpath.Replace('/', '\\');
            steaminstallpath += "common\\quadrilateralcowboy\\";

            if (Directory.Exists(steaminstallpath))
            {
                return steaminstallpath;
            }

            return string.Empty;
        }


        private void button2_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog diag = new FolderBrowserDialog();
            diag.Description = "Select the mod's folder.";
            diag.ShowNewFolderButton = false;
            diag.RootFolder = Environment.SpecialFolder.MyComputer;

            string qcFolder = GetQCFolder();
            if (Directory.Exists(qcFolder))
            {
                diag.SelectedPath = qcFolder;
            }

            DialogResult result = diag.ShowDialog();

            //hack to select the folder.
            //SendKeys.Send("{TAB}{TAB}{RIGHT}");



            if (DialogResult.OK == result)
            {
                textbox_data.Text = diag.SelectedPath;
            }

            diag.Dispose();
        }

        private void label5_Click(object sender, EventArgs e){}

        private void checkedListBox1_SelectedIndexChanged(object sender, EventArgs e){}

        private void loadingpic_Click(object sender, EventArgs e){}

        private void combobox_visibility_SelectedIndexChanged(object sender, EventArgs e) { }

        //allow only one to be checked.
        private void checkedListBox1_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            for (int ix = 0; ix < listbox_tags.Items.Count; ++ix)
            {
                if (ix != e.Index) listbox_tags.SetItemChecked(ix, false);
            }
        }



        //Verify the data works for QC. Change this if game is not QC
        private bool VerifyDataFiles()
        {
            if (!Directory.Exists(textbox_data.Text))
            {
                MessageBox.Show(string.Format("Cannot find data folder:\n{0}", textbox_data.Text), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            if (!File.Exists(textbox_data.Text + "\\description.txt"))
            {
                MessageBox.Show("Data folder is missing description.txt", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            string[] pakFiles = System.IO.Directory.GetFiles(textbox_data.Text, "*.pk4", System.IO.SearchOption.TopDirectoryOnly);
            if (pakFiles.Length <= 0)
            {
                MessageBox.Show("Data folder is missing a PK4 file.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            string[] dllFiles = Directory.GetFiles(textbox_data.Text, "*.dll", SearchOption.AllDirectories);

            if (dllFiles.Length > 0)
            {
                string dllNames = string.Empty;
                for (int i = 0; i < dllFiles.Length; i++)
                {
                    dllNames += string.Format("{0}{1}", dllFiles[i], Environment.NewLine);
                }

                MessageBox.Show( string.Format("DLL files are not allowed.\n\nPlease delete the following files before uploading:\n{0}", dllNames), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }

        private bool VerifyFields()
        {
            if (string.IsNullOrWhiteSpace(textbox_name.Text))
            {
                MessageBox.Show("'Name' field cannot be empty.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }

        //this function is to auto scroll the debug log.
        void SetDebugLog(string text)
        {
            textbox_debug.Items.Add(text);            

            int nItems = (int)(textbox_debug.Height / textbox_debug.ItemHeight);
            textbox_debug.TopIndex = textbox_debug.Items.Count - nItems;        
        }


        private void PublishEnd()
        {
            SetAllControlsEnabled(true);

            DateTime end = DateTime.Now;
            TimeSpan delta = end.Subtract(startTime);

            SetDebugLog(string.Format("Total time: {0:0.#} seconds", delta.TotalSeconds));
            loadingpic.Hide();
        }

        private void SetAllControlsEnabled(bool value)
        {
            foreach (Control control in this.Controls)
            {
                if (string.Compare(control.Name, "loadingpic", true) == 0)
                    continue;

                control.Enabled = value;
            }
        }

        bool GetAreTagsDifferent(List<string> tagList)
        {
            if (new HashSet<string>(tagList).SetEquals(originalInfo.tags))
                return false;
            else
                return true;
        }








        // ===========================================================================

        //press the COMMIT button.

        // ===========================================================================

        PublishedFileId_t publishedFileID;
        private CallResult<CreateItemResult_t> OnCreateItemResultCallResult;
        private CallResult<SubmitItemUpdateResult_t> OnSubmitItemUpdateResultCallResult;
        private UGCUpdateHandle_t m_UGCUpdateHandle;

        private void button_publishmod_Click(object sender, EventArgs e)
        {
            startTime = DateTime.Now;
            SetAllControlsEnabled(false);
            loadingpic.Show();

            SetDebugLog("----------------------------------------------------------------------");
            SetDebugLog("Verifying mod data...");
            if (!string.IsNullOrWhiteSpace(textbox_data.Text))
            {

                //player is uploading new data. Verify it includes the correct files.
                if (!VerifyDataFiles())
                {
                    SetDebugLog("Aborting upload.");
                    PublishEnd();                    
                    return;
                }

                if (!VerifyFields())
                {
                    SetDebugLog("Aborting upload.");
                    PublishEnd();                    
                    return;
                }
            }

            if (IsCreating)
            {
                //Creating new mod.
                SetDebugLog("Creating Workshop mod...");
                OnCreateItemResultCallResult = CallResult<CreateItemResult_t>.Create(OnCreateItemResult);
                SteamAPICall_t handle = SteamUGC.CreateItem(Constants.GetAppID(), EWorkshopFileType.k_EWorkshopFileTypeCommunity);
                OnCreateItemResultCallResult.Set(handle);
            }
            else
            {
                //updating an existing mod.
                SetItemData();
            }
        }

        void OnCreateItemResult(CreateItemResult_t pCallback, bool bIOFailure)
        {
            if (bIOFailure) {SetDebugLog(string.Format("bIOFailure: true"));}

            SetDebugLog(string.Format("Callback: {0} ", pCallback.m_eResult));

            if (pCallback.m_bUserNeedsToAcceptWorkshopLegalAgreement) {SetDebugLog(string.Format("USER NEEDS TO ACCEPT STEAM WORKSHOP LEGAL AGREEMENT: true"));}

            switch (pCallback.m_eResult)
            {
                case EResult.k_EResultTimeout:
                    MessageBox.Show("The operation took longer than expected. Please retry again.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    PublishEnd();
                    return;
                case EResult.k_EResultNotLoggedOn:
                    MessageBox.Show("You are not logged into Steam. Please log in to Steam.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    PublishEnd();
                    return;
                case EResult.k_EResultInsufficientPrivilege:
                    MessageBox.Show("You do not have workshop privileges.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    PublishEnd();
                    return;
                case EResult.k_EResultOK:
                    break;
                default:
                    MessageBox.Show(string.Format("There was a problem creating workshop mod.\n\nError code: {0}", pCallback.m_eResult), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    PublishEnd();
                    return;
            }

            publishedFileID = pCallback.m_nPublishedFileId;
            SetDebugLog(string.Format("Publisher File ID: {0}", pCallback.m_nPublishedFileId));
            SetDebugLog("Workshop mod created.");

            SetItemData();
        }

        void SetItemData()
        {
            SetDebugLog("Setting mod data...");
            m_UGCUpdateHandle = SteamUGC.StartItemUpdate(Constants.GetAppID(), publishedFileID);

            bool changed = false;

            //Title.
            if (string.Compare(originalInfo.name, textbox_name.Text) != 0 && !string.IsNullOrWhiteSpace(textbox_name.Text))
            {
                SetDebugLog(string.Format("Updating mod name. ({0})", textbox_name.Text));
                SteamUGC.SetItemTitle(m_UGCUpdateHandle, textbox_name.Text);
                changed = true;
            }

            //Description.
            if (string.Compare(originalInfo.description, textbox_description.Text) != 0 && !string.IsNullOrWhiteSpace(textbox_description.Text))
            {
                SetDebugLog("Updating mod description.");
                SteamUGC.SetItemDescription(m_UGCUpdateHandle, textbox_description.Text);
                changed = true;
            }

            //Thumbnail.
            if (!string.IsNullOrWhiteSpace(textbox_thumbnail.Text))
            {
                SetDebugLog(string.Format("Updating mod thumbnail. ({0})", textbox_thumbnail.Text));
                SteamUGC.SetItemPreview(m_UGCUpdateHandle, textbox_thumbnail.Text);
                changed = true;
            }

            //Data.
            if (!string.IsNullOrWhiteSpace(textbox_data.Text))
            {
                SetDebugLog(string.Format("Updating mod data. ({0})", textbox_data.Text));
                SteamUGC.SetItemContent(m_UGCUpdateHandle, textbox_data.Text);
                changed = true;
            }

            //Visibility.
            if (combobox_visibility.SelectedIndex != originalInfo.visibility)
            {
                SetDebugLog(string.Format("Updating mod visibility. ({0})", combobox_visibility.SelectedIndex));

                ERemoteStoragePublishedFileVisibility visibility;
                switch (combobox_visibility.SelectedIndex)
                {
                    case 0:
                        visibility = ERemoteStoragePublishedFileVisibility.k_ERemoteStoragePublishedFileVisibilityPublic;
                        break;
                    case 1:
                        visibility = ERemoteStoragePublishedFileVisibility.k_ERemoteStoragePublishedFileVisibilityFriendsOnly;
                        break;
                    default:
                        visibility = ERemoteStoragePublishedFileVisibility.k_ERemoteStoragePublishedFileVisibilityPrivate;
                        break;
                }
                SteamUGC.SetItemVisibility(m_UGCUpdateHandle, visibility);
                changed = true;
            }



            //Tags.
            if (listbox_tags.CheckedItems.Count > 0)
            {
                List<string> tagList = new List<string>();
                for (int i = 0; i < listbox_tags.CheckedItems.Count; i++)
                {
                    tagList.Add(listbox_tags.CheckedItems[i].ToString());
                }

                //do a diff on the tags.
                if (GetAreTagsDifferent(tagList))
                {
                    SetDebugLog("Updating mod tags.");
                    SteamUGC.SetItemTags(m_UGCUpdateHandle, tagList);
                    changed = true;
                }                
            }


            

            if (!changed)
            {
                PublishEnd();
                MessageBox.Show("No changes found. Stopping update.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);                
                return;
            }
            
            


            SetDebugLog(" ");
            SetDebugLog("Uploading Workshop mod, please wait...");
            OnSubmitItemUpdateResultCallResult = CallResult<SubmitItemUpdateResult_t>.Create(OnSubmitItemUpdateResult);
            SteamAPICall_t handle = SteamUGC.SubmitItemUpdate(m_UGCUpdateHandle, textbox_notes.Text);
            OnSubmitItemUpdateResultCallResult.Set(handle);
        }

        void OnSubmitItemUpdateResult(SubmitItemUpdateResult_t pCallback, bool bIOFailure)
        {
            if (bIOFailure) { SetDebugLog(string.Format("bIOFailure: true")); }

            SetDebugLog(string.Format("Callback: {0}",  pCallback.m_eResult));

            if (pCallback.m_bUserNeedsToAcceptWorkshopLegalAgreement) { SetDebugLog(string.Format("USER NEEDS TO ACCEPT STEAM WORKSHOP LEGAL AGREEMENT: true")); }

            PublishEnd();
            

            if (pCallback.m_eResult != EResult.k_EResultOK)
            {
                MessageBox.Show(string.Format("There was a problem uploading the workshop mod.\n\nError code: {0}", pCallback.m_eResult), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                SetDebugLog("Done. Successful upload.");

                if (IsCreating)
                {
                    MessageBox.Show(string.Format("Published new mod!"), string.Empty, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    //System.Diagnostics.Process.Start(string.Format("http://steamcommunity.com/sharedfiles/filedetails/?id={0}", publishedFileID));
                    this.Close();
                }
                else
                {
                    //synchronize the OriginalInfo modinfo to what is currently in the fields.

                    originalInfo.name = textbox_name.Text;
                    originalInfo.description = textbox_description.Text;
                    originalInfo.visibility = combobox_visibility.SelectedIndex;

                    originalInfo.tags = new List<string>();
                    for (int k = 0; k < listbox_tags.Items.Count; k++)
                    {
                        if (listbox_tags.GetItemChecked(k))
                        {
                            string listTag = listbox_tags.GetItemText(listbox_tags.Items[k]);
                            originalInfo.tags.Add(listTag);
                        }
                    }

                    MessageBox.Show(string.Format("Update published!"), string.Empty, MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            } 
        }




    }
}
